@@ -1,6 +1,6 @@
 package Provision::Unix::VirtualOS::Linux::Xen;
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
 
 use warnings;
 use strict;
@@ -51,7 +51,7 @@ sub create_virtualos {
         debug   => $vos->{debug},
     ) if $self->is_present();
 
-    $self->_is_valid_template( $vos->{template} )
+    $self->is_valid_template( $vos->{template} )
         or return $prov->error(
         message => "no valid template specified",
         fatal   => $vos->{fatal},
@@ -69,7 +69,7 @@ sub create_virtualos {
     $self->extract_template()
         or $prov->error(
         message => "unable to extract template onto disk image" );
-    $self->_install_config_file()
+    $self->install_config_file()
         or $prov->error( message => "unable to install config file" );
 
     # TODO:
@@ -111,19 +111,20 @@ sub destroy_virtualos {
     $self->destroy_disk_image();
     $self->destroy_swap_image();
 
-    my $container_dir = "$vos->{disk_root}/$vos->{name}.vm";
+    my $container_dir = $self->get_ve_home() or
+        $prov->error( message => "could not deduce the containers home dir" );
+
+    return 1 if ! -d $container_dir;
+
+    my $cmd = $util->find_bin( bin => 'rm', debug => 0 );
+    $prov->audit("$cmd -rf $container_dir");
+    $util->syscmd(
+        cmd   => "$cmd -rf $container_dir",
+        debug => $vos->{debug},
+        fatal => $vos->{fatal},
+    );
     if ( -d $container_dir ) {
-        my $cmd = $util->find_bin( bin => 'rm', debug => 0 );
-        $prov->audit("$cmd -rf $container_dir");
-        $util->syscmd(
-            cmd   => "$cmd -rf $container_dir",
-            debug => 0,
-            fatal => $vos->{fatal}
-        );
-        if ( -d $container_dir ) {
-            $prov->error( message => "failed to delete $container_dir" );
-        }
-        return 1;
+        $prov->error( message => "failed to delete $container_dir" );
     }
     return 1;
 }
@@ -140,7 +141,7 @@ sub start_virtualos {
         debug   => $vos->{debug},
     ) if !$self->is_present();
 
-    my $config_file = $self->get_config_file_path();
+    my $config_file = $self->get_ve_config_path();
     if ( !-e $config_file ) {
         $prov->error( message =>
                 "config file for $ctid should be at $config_file and is missing."
@@ -207,7 +208,7 @@ sub disable_virtualos {
     ) if !$self->is_present();
 
     # make sure config file exists
-    my $config = $self->get_config_file_path();
+    my $config = $self->get_ve_config_path();
     if ( !-e $config ) {
         return $prov->error(
             message =>
@@ -255,7 +256,7 @@ sub enable_virtualos {
     ) if $self->is_enabled();
 
     # make sure config file exists
-    my $config = $self->get_config_file_path();
+    my $config = $self->get_ve_config_path();
     if ( !-e "$config.suspend" ) {
         return $prov->error(
             message =>
@@ -330,13 +331,14 @@ sub destroy_disk_image {
     $prov->audit("checking for presense of $img_name");
     if ( -e "/dev/vol00/$img_name" ) {
         $prov->audit("\tfound it. You killed my father, prepare to die!");
-        my $cmd
-            = $util->find_bin( bin => 'lvremove', debug => $vos->{debug} );
+        my $cmd = $util->find_bin( bin => 'lvremove', debug => $vos->{debug} );
         $cmd .= " -f vol00/${img_name}";
         $prov->audit("\tcmd: $cmd");
         $util->syscmd( cmd => $cmd, debug => 0 )
-            or $prov->error( message => "unable to destroy $img_name" );
+            or return $prov->error( message => "unable to destroy $img_name" );
+        return 1;
     }
+    return;
 }
 
 sub destroy_swap_image {
@@ -350,14 +352,16 @@ sub destroy_swap_image {
         $cmd .= " -f vol00/${img_name}";
         $prov->audit("\tcmd: $cmd");
         $util->syscmd( cmd => $cmd, debug => 0 )
-            or $prov->error( message => "unable to destroy $img_name" );
+            or return $prov->error( message => "unable to destroy $img_name" );
+        return 1;
     }
+    return;
 }
 
 sub extract_template {
     my $self = shift;
 
-    $self->_is_valid_template( $vos->{template} )
+    $self->is_valid_template( $vos->{template} )
         or return $prov->error(
         message => "no valid template specified",
         fatal   => $vos->{fatal},
@@ -367,7 +371,7 @@ sub extract_template {
     $self->mount_disk_image();
 
     my $mount_dir    = "$vos->{disk_root}/$vos->{name}.vm/mnt";
-    my $template_dir = $prov->{config}{VirtualOS}{xen_template_dir};
+    my $template_dir = $self->get_template_dir();
 
     #tar -zxf $template_dir/$OSTEMPLATE.tar.gz -C /home/xen/${VMNAME}.vm/mnt
 
@@ -382,6 +386,203 @@ sub extract_template {
     $self->unmount_disk_image();
 }
 
+sub get_random_mac {
+
+    my $i;
+    my $lladdr = '00:16:3E';
+
+    while ( ++$i ) {
+        last if $i > 6;
+        $lladdr .= ':' if $i % 2;
+        $lladdr .= sprintf "%" . ( qw (X x) [ int( rand(2) ) ] ),
+            int( rand(16) );
+    }
+
+    # TODO:
+    #   make sure random MAC does not conflict with an existing one.
+
+    return $lladdr;
+}
+
+sub get_status {
+    my $self = shift;
+
+    my $name = $vos->{name};   # the vm/container we're interested in
+
+    my $cmd = $util->find_bin( bin => 'xm', debug => 0 );
+    $cmd .= ' list';
+    my $r = `$cmd`;
+    $r =~ /VCPUs/
+        or return $prov->error(
+        message => 'could not get valid output from xm list' );
+
+    my ($xen_conf, $config);
+    eval "require Provision::Unix::VirtualOS::Xen::Config";
+    if ( ! $EVAL_ERROR ) {
+        $xen_conf = Provision::Unix::VirtualOS::Xen::Config->new();
+    };
+
+    $self->{status} = {};
+    foreach my $line ( split /\n/, $r ) {
+
+ # Name                                      ID Mem(MiB) VCPUs State   Time(s)
+ #Domain-0                                   0     1024     4 r-----  62976.7
+ #test                                      20       63     1 -b----     34.1
+
+        my ( $ctid, $dom_id, $mem, $cpus, $state, $time ) = split /\s+/, $line;
+        next unless $ctid;
+        next if $ctid eq 'Name';
+        next if $ctid eq 'Domain-0';
+        next if ( $ctid ne $name && $ctid ne "$name.vm" );
+
+        my ($ips, $disks );
+
+        # find the containers config file
+        my $config_file = $self->get_ve_config_path();
+
+        if ( -e $config_file ) {
+            if ( $xen_conf && $xen_conf->read_config($config_file) ) {
+                $ips   = $xen_conf->get_ips();
+                $disks = $xen_conf->get('disk');
+            };
+        }
+        else {
+            warn "could not find $config_file\n";
+        };
+
+        $self->{status}{$ctid} = {
+            ips      => $ips,
+            disks    => $disks,
+            dom_id   => $dom_id,
+            mem      => $mem + 1,
+            cpus     => $cpus,
+            state    => _run_state($state),
+            cpu_time => $time,
+        };
+        return $self->{status}{$ctid};
+    }
+
+    #warn Dumper $self->{status};
+
+    sub _run_state {
+        my $abbr = shift;
+        return
+              $abbr =~ /r/ ? 'running'
+            : $abbr =~ /b/ ? 'running' # blocked is a 'wait' state, poorly named
+            : $abbr =~ /p/ ? 'paused'
+            : $abbr =~ /s/ ? 'shutdown'
+            : $abbr =~ /c/ ? 'crashed'
+            : $abbr =~ /d/ ? 'dying'
+            :                undef;
+    }
+}
+
+sub get_template_dir {
+    my $self = shift;
+
+    my $template_dir = $prov->{config}{VirtualOS}{xen_template_dir} || '/templates';
+    return $template_dir;
+};
+
+sub get_ve_config_path {
+    my $self = shift;
+    my $ctid = $vos->{name};
+    my $config_file = "$vos->{disk_root}/$ctid.vm/$ctid.vm.cfg";
+    return $config_file;
+};
+
+sub get_ve_home {
+    my $self = shift;
+    my $ctid = $vos->{name};
+    my $homedir = "$vos->{disk_root}/$ctid.vm";
+    return $homedir;
+};
+
+sub is_present {
+    my $self = shift;
+
+    my %p = validate(
+        @_,
+        {   'name'    => { type => SCALAR, optional => 1 },
+            'refresh' => { type => SCALAR, optional => 1, default => 1 },
+        }
+    );
+
+    my $name = $p{name} || $vos->{name} or
+        $prov->error( message => 'is_present was called without a CTID' );
+
+    $prov->audit("checking for presense of virtual container $name");
+
+    my @possible_paths = (
+        "$vos->{disk_root}/$name.vm",
+        "/dev/vol00/test1_rootimg", "/dev/vol00/test1_vmswap"
+    );
+
+    foreach my $path (@possible_paths) {
+        $prov->audit("\tchecking at $path");
+        if ( -e $path ) {
+            $prov->audit("\tfound $name at $path");
+            return $path;
+        }
+    }
+
+    $prov->audit("\tdid not find $name");
+    return;
+}
+
+sub is_running {
+
+    my $self = shift;
+
+    my %p = validate(
+        @_,
+        {   'name'    => { type => SCALAR, optional => 1 },
+            'refresh' => { type => SCALAR, optional => 1, default => 1 },
+            'test_mode' => { type => SCALAR | UNDEF, optional => 1 },
+            'debug' => { type => BOOLEAN, optional => 1, default => 1 },
+            'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
+        }
+    );
+
+    my $name = $p{name} || $vos->{name}
+        or $prov->error( message => 'is_running was called without a CTID' );
+
+    $self->get_status() if $p{refresh};
+
+    if ( $self->{status}{"$name.vm"} ) {
+        my $state = $self->{status}{"$name.vm"}{state};
+        return 1 if ( $state && $state eq 'running' );
+    }
+    return;
+}
+
+sub is_enabled {
+    my $self = shift;
+
+    my %p = validate(
+        @_,
+        {   'name' =>
+                { type => SCALAR, optional => 1, default => $vos->{name} },
+            'debug' => { type => BOOLEAN, optional => 1, default => 1 },
+            'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
+        }
+    );
+
+    my $name = $p{name}
+        || $prov->error( message => 'is_enabled was called without a CTID' );
+    $prov->audit("testing if virtual container $name is enabled");
+
+    my $config = "$vos->{disk_root}/$name.vm/$name.cfg";
+
+    if ( -e $config ) {
+        $prov->audit("\tfound $name at $config");
+        return 1;
+    }
+
+    $prov->audit("\tdid not find $config");
+    return;
+}
+
 sub mount_disk_image {
     my $self = shift;
 
@@ -432,210 +633,16 @@ sub unmount_disk_image {
         or $prov->error( message => "unable to unmount $img_name" );
 }
 
-sub get_status {
-    my $self = shift;
-
-    my $cmd = $util->find_bin( bin => 'xm', debug => 0 );
-    $cmd .= ' list';
-    my $r = `$cmd`;
-    $r =~ /VCPUs/
-        or return $prov->error(
-        message => 'could not get valid output from xm list' );
-
-    my ($xen_conf, $config);
-    eval "require Provision::Unix::VirtualOS::Xen::Config";
-    if ( ! $EVAL_ERROR ) {
-        $xen_conf = Provision::Unix::VirtualOS::Xen::Config->new();
-    };
-
-    $self->{status} = {};
-    foreach my $line ( split /\n/, $r ) {
-
- # Name                                      ID Mem(MiB) VCPUs State   Time(s)
- #Domain-0                                   0     1024     4 r-----  62976.7
- #test                                      20       63     1 -b----     34.1
-
-        my ( $ctid, $dom_id, $mem, $cpus, $state, $time ) = split /\s+/,
-            $line;
-        next unless $ctid;
-        next if $ctid eq 'Name';
-        next if $ctid eq 'Domain-0';
-        my ($ips, $disks );
-        my $container_config = "/etc/xen/auto/$ctid.cfg";
-        if ( ! -f $container_config ) {
-            $container_config = "/etc/xen/$ctid/$ctid.cfg";
-        };
-        if ( -f $container_config ) {
-            if ( $xen_conf && $xen_conf->read_config($container_config) ) {
-                $ips = $xen_conf->get_ips();
-                $disks = $xen_conf->get('disk');
-            };
-        }
-        else {
-            warn "could not find $container_config\n";
-        };
-
-        $self->{status}{$ctid} = {
-            ips      => $ips,
-            disks    => $disks,
-            dom_id   => $dom_id,
-            mem      => $mem,
-            cpus     => $cpus,
-            status   => _run_state($state),
-            'time'   => $time,
-        };
-    }
-
-    #warn Dumper $self->{status};
-
-    my $name = $vos->{name};
-    if ( $name ) {
-        if ( $self->{status}{ $name } ) {
-            return $self->{status}{ $name };
-        };
-        if ( $self->{status}{ "$name.vm" } ) {
-            return $self->{status}{ "$name.vm" };
-        };
-        warn "unable to find vm $name\n";
-    }
-    return $self->{status};
-
-    sub _run_state {
-        my $abbr = shift;
-        return
-              $abbr =~ /r/ ? 'running'
-            : $abbr =~ /b/ ? 'blocked'
-            : $abbr =~ /p/ ? 'paused'
-            : $abbr =~ /s/ ? 'shutdown'
-            : $abbr =~ /c/ ? 'crashed'
-            : $abbr =~ /d/ ? 'dying'
-            :                undef;
-    }
-}
-
-sub is_present {
-    my $self = shift;
-
-    my %p = validate(
-        @_,
-        {   'name' =>
-                { type => SCALAR, optional => 1, default => $vos->{name} },
-            'refresh' => { type => SCALAR, optional => 1, default => 1 },
-            'test_mode' => { type => SCALAR | UNDEF, optional => 1 },
-            'debug' => { type => BOOLEAN, optional => 1, default => 1 },
-            'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
-        }
-    );
-
-    my $name = $p{name};
-
-    $name || $prov->error( message => 'is_present was called without a CTID' );
-    $prov->audit("checking for presense of virtual container $name");
-
-    my @possible_paths = (
-        "$vos->{disk_root}/$name.vm",
-        "/dev/vol00/test1_rootimg", "/dev/vol00/test1_vmswap"
-    );
-
-    foreach my $path (@possible_paths) {
-        $prov->audit("\tchecking at $path");
-        if ( -e $path ) {
-            $prov->audit("\tfound $name at $path");
-            return $path;
-        }
-    }
-
-    $prov->audit("\tdid not find $name");
-    return;
-}
-
-sub is_running {
-
-    my $self = shift;
-
-    my %p = validate(
-        @_,
-        {   'name' =>
-                { type => SCALAR, optional => 1, default => $vos->{name} },
-            'refresh' => { type => SCALAR, optional => 1, default => 1 },
-            'test_mode' => { type => SCALAR | UNDEF, optional => 1 },
-            'debug' => { type => BOOLEAN, optional => 1, default => 1 },
-            'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
-        }
-    );
-
-    $p{name}
-        || $prov->error( message => 'is_running was called without a CTID' );
-    $self->get_status() if $p{refresh};
-    if ( $self->{status}{"$p{name}.vm"} ) {
-        my $state = $self->{status}{"$p{name}.vm"}{status};
-        return 1 if ( $state eq 'blocked' || $state eq 'running' );
-    }
-    return;
-}
-
-sub is_enabled {
-    my $self = shift;
-
-    my %p = validate(
-        @_,
-        {   'name' =>
-                { type => SCALAR, optional => 1, default => $vos->{name} },
-            'debug' => { type => BOOLEAN, optional => 1, default => 1 },
-            'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
-        }
-    );
-
-    my $name = $p{name}
-        || $prov->error( message => 'is_enabled was called without a CTID' );
-    $prov->audit("testing if virtual container $name is enabled");
-
-    my $config = "$vos->{disk_root}/$name.vm/$name.cfg";
-
-    if ( -e $config ) {
-        $prov->audit("\tfound $name at $config");
-        return 1;
-    }
-
-    $prov->audit("\tdid not find $config");
-    return;
-}
-
-sub _get_random_mac {
-
-    my $i;
-    my $lladdr = '00:16:3E';
-
-    while ( ++$i ) {
-        last if $i > 6;
-        $lladdr .= ':' if $i % 2;
-        $lladdr .= sprintf "%" . ( qw (X x) [ int( rand(2) ) ] ),
-            int( rand(16) );
-    }
-
-    # TODO:
-    #   make sure random MAC does not conflict with an existing one.
-
-    return $lladdr;
-}
-
-sub get_config_file_path {
-    my $self = shift;
-    my $ctid = $vos->{name};
-    my $config_file = "$vos->{disk_root}/$ctid.vm/$ctid.vm.cfg";
-    return $config_file;
-};
-
-sub _install_config_file {
+sub install_config_file {
     my $self = shift;
 
     my $ctid        = $vos->{name};
-    my $config_file = $self->get_config_file_path();
+    my $config_file = $self->get_ve_config_path();
     warn "config file: $config_file\n";
 
     my $ip       = $vos->{ip}->[0];
     my $ram      = $vos->{ram} || 64;
-    my $mac      = $self->_get_random_mac();
+    my $mac      = $self->get_random_mac();
     my $hostname = $vos->{hostname} || $ctid;
 
     my $config = <<"EOCONF"
@@ -663,12 +670,12 @@ EOCONF
     return 1;
 }
 
-sub _is_valid_template {
+sub is_valid_template {
 
     my $self = shift;
     my $template = shift or return;
 
-    my $template_dir = $prov->{config}{VirtualOS}{xen_template_dir};
+    my $template_dir = $self->get_template_dir();
     if ( -f "$template_dir/$template.tar.gz" ) {
         return $template;
     }
@@ -708,7 +715,6 @@ Provision::Unix::VirtualOS::Linux::Xen - Provision Xen containers
 =head1 SYNOPSIS
 
 
-
 =head1 FUNCTIONS
 
 
